#!/bin/bash 
# chkconfig: 3 06 94
# description: Performs BIOS recovery


log=/var/log/messages
bios_dir=/opt/tms/bin
bios_img=bios.rom

iorw_util=/opt/tms/bin/iorw
lpc_base=0x2500
ci_kgi_offs=0x2b

count_file=/var/opt/tms/output/bios_recovery_count
count=0
max_count=3

log_output()
{
	date_time=$(date | awk '{print $2" " $3 " " $4" "}')
	echo -e ${date_time}"\t"$1 | tee -a $log 
}

check_ci_kgi_boot()
{
	lpc_mod=$(lsmod | grep lpci2c)
	len=${#lpc_mod}
	if [ $len -lt 6 ]; then
		log_output "Error: lpci2c module is not loaded. Can not check CI/KGI boot"
		return 1
	else
		let "ci_kgi_reg=$lpc_base+$ci_kgi_offs"
		ci_kgi=$($iorw_util -r -b $ci_kgi_reg -l 1 | awk '{print $5}')
		let "ci_kgi>>=3"
		let "ci_kgi&=1"

		if [ $ci_kgi -eq 1 ]; then
			log_output "Error: System is booted from KGI SPI flash!\n"
			return 2
		else
			log_output "System is booted from CI SPI flash"
			return 0
		fi	
	fi
}

check_recovery_count()
{
	if [ ! -e $count_file ]; then
		echo 0 > $count_file
		return 0
	else
		count=$(cat $count_file)
		return $count
	fi
}

update_bios()
{
	/opt/tms/bin/bios_update.sh $1
	rc=$?
	if [ $rc -ne 0 ]; then
		log_output "Error: BIOS update failed"
		return 1
	else
		log_output "BIOS update success"
		return 0
	fi
}

start()
{
	check_ci_kgi_boot
	rc=$?
	if [ $rc -eq 0 ]; then
	#System is booted from CI, reset counter
		check_recovery_count
		if [ $? -ne 0 ]; then
			echo 0 > $count_file
		fi
		return 0
	else
		#Remove following line after fix in CPLD
		exit 1
		if [ $rc -gt 1 ]; then
			check_recovery_count
			count=$?
			if [ $count -ge 3 ]; then
				log_output "Error: Unrecoverable BIOS SPI problem!"
				return 1
			else
				let count+=1
				echo $count > $count_file
				update_bios "${bios_dir}/${bios_img}"
			fi
		else
			log_output "Error: can not perform recovery check"
			return 1
		fi
	fi
}

stop()
{
	check_recovery_count
	count=$?
	if [ $count -gt 0 ]; then
		log_output "${count} BIOS recovery attempts have been done"
	fi
}

case "$1" in
  start)
        start
        ;;
  stop)
        stop
        ;;
  restart)
        stop
        start
        ;;
  status)
        ;;
  *)
        echo $"Usage: $0 {start|stop|restart}"
        exit 1
esac

exit $?
