/**
 *
 * Copyright (C) 2012-2014, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */
#ifndef __CPLD_HANDLER_H_
#define __CPLD_HANDLER_H_

#include <mlnx_log.h>
#include <mlnx_irq.h>

/* See:
   DxIR Device x Interrupt Route Register
   Table 5-16. APIC Interrupt Mapping
*/
#define PIRQB 17 /*PCH ball name PIRQB#, ball K38*/
#define PIRQC 18 /*PCH ball name PIRQC#, ball H38*/
#define PIRQD 19 /*PCH ball name PIRQD#, ball G38*/

typedef int (*cpld_handler_action)(void *subscriber_data,
					struct int_digest_blk_t *p_digest_blk,
					struct mod_digest *digest_entry,
					int module_ind,
					int level);
struct cpld_handler_action_entry {
	struct list_head action_list_1;              // Container embedded action list for level 1
	struct list_head action_list_2;              // Container embedded action list for level 2
	struct list_head action_list_3;              // Container embedded action list for level 3
	struct list_head action_list_ha;             // Container embedded action list for level ha
	struct list_head action_list_4;              // Container embedded action list for level 4 (replacable module level)
	struct list_head action_list_5;              // Container embedded action list for level 5 (replacable module level)
	cpld_handler_action pre_action;              // Pre-action to be executed
	cpld_handler_action post_action;             // Post-action to be executed
	void *subscriber_data;                       // User data
	int index;                                   // Device interrupt line id
	int usr_handle;                              // Upper user hanlde
	enum irq_level_e int_level;                  // Interrupt's hierarchical level
	u32 int_digest;                              // Module's interrupt mask
	u16 event_checksum;                          // Checksum: if 0 - no raised interrupts detected
	u16 status_checksum;                         // Checksum: if 0 - no status change detected
	u8 aggregation;                              // Aggregation: if 1 - status change detected, no event exists
	u8 recovered;                                // Recovered after transaction failure
	u8 termination;                              // Entry is interrupt termination point
	struct mod_digest *digest_entry;             // Digest entry
};
extern int cpld_handler_action_subscribe(struct cpld_handler_action_entry *action_entry);
extern int cpld_handler_action_unsubscribe(struct cpld_handler_action_entry *action_entry);

#endif /* __CPLD_HANDLER_H_ */
 
