/*
* Copyright (C) Mellanox Technologies, Ltd. 2001-2014. ALL RIGHTS RESERVED.
*
* Feb 25, 2014
* This software product is a proprietary product of Mellanox Technologies, Ltd.
* (the "Company") and all right, title, and interest in and to the software product,
* including all associated intellectual property rights, are and shall
* remain exclusively with the Company.
*
* This software product is governed by the End User License Agreement
* provided with the software product.
*
*/

#define I2C_NAME_SIZE   20 // got from <linux/mod_devicetable.h>

#include <stdint.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <linux/i2c-dev.h>
#include "linux/i2c.h"
#include "mlnx_if.h"
#include "mlnx_if_us.h"

#define SYSFS_PATH_RAW "/sys/bus/i2c/devices/%d-%04x"
#define SYSFS_PATH_ATTR "/sys/bus/i2c/devices/%d-%04x/%s"
#define DEV_PATH "/dev/%s"

static int mellaggra_dbg = 0;
#define DBG(args...) {\
	if (mellaggra_dbg) {\
                printf(args);\
	}\
}

#define VPD_WR_DELAY (10000)

static void i2c_translate_offset(u32 offset, int addr_width, unsigned char *msgbuf)
{
	int i = 0;

	switch (addr_width) {

	case 4:
		msgbuf[i++] = offset >> 24;
		msgbuf[i++] = offset >> 16;
		msgbuf[i++] = offset >> 8;
		msgbuf[i] = offset;
		break;
	case 2:
		offset &= 0xffff;
		msgbuf[i++] = offset >> 8;
		msgbuf[i] = offset;
		break;
	case 1:
		offset &= 0xff;
		msgbuf[i] = offset;
		break;
	}
}
/* Emulation of i2c transaction via SMBus.
#if defined(__i386__) || defined(__x86_64__)

int mlnx_i2c_read(int bus, u16 addr, u32 offset, size_t addr_width, char *data, size_t datalen)
{
	struct mlnx_rdwr_smbus_ioctl_params_2 msg_rdwr_smbus;
	char path[256];
	int fd;
	int rc;

	if (datalen > I2C_BLOCK_MAX)
		return -ENOSPC;
	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	memset(&msg_rdwr_smbus, 0, sizeof(struct mlnx_rdwr_smbus_ioctl_params_2));
	msg_rdwr_smbus.bus = bus;
	msg_rdwr_smbus.addr_width = addr_width;
	msg_rdwr_smbus.addr = addr;
	msg_rdwr_smbus.offset = offset;
	msg_rdwr_smbus.read_write = I2C_SMBUS_READ;
	msg_rdwr_smbus.datalen = datalen;
	rc = ioctl(fd, IOCTL_MLNX_SMBUS_RW, &msg_rdwr_smbus);
	close(fd);
	memcpy(data, msg_rdwr_smbus.data, datalen);
	return rc >= 0 ? 0 : rc;
}

int mlnx_i2c_write(int bus, u16 addr, u32 offset, size_t addr_width, char *data, size_t datalen)
{
	struct mlnx_rdwr_smbus_ioctl_params_2 msg_rdwr_smbus;
	char path[256];
	int fd;
	int rc;

	if (datalen > I2C_BLOCK_MAX)
		return -ENOSPC;
	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	memset(&msg_rdwr_smbus, 0, sizeof(struct mlnx_rdwr_smbus_ioctl_params_2));
	msg_rdwr_smbus.bus = bus;
	msg_rdwr_smbus.addr_width = addr_width;
	msg_rdwr_smbus.addr = addr;
	msg_rdwr_smbus.offset = offset;
	msg_rdwr_smbus.read_write = I2C_SMBUS_WRITE;
	msg_rdwr_smbus.datalen = datalen;
	memcpy(msg_rdwr_smbus.data, data, datalen);
	rc = ioctl(fd, IOCTL_MLNX_SMBUS_RW, &msg_rdwr_smbus);
	close(fd);
	return rc >= 0 ? 0 : rc;
}

#else
*/
int mlnx_i2c_read(int bus, u16 addr, u32 offset, size_t addr_width, char *data, size_t datalen)
{
	unsigned char msgbuf0[4];
	unsigned char msgbuf1[I2C_BLOCK_MAX];
	struct mlnx_rdwr_i2c_ioctl_params msg_rdwr;
	char path[256];
	int fd;
	int rc;

	if (datalen > I2C_BLOCK_MAX)
		return -ENOSPC;
	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);

	memset(&msg_rdwr, 0, sizeof(struct mlnx_rdwr_i2c_ioctl_params));
	memset(msgbuf1, 0, sizeof(struct i2c_rdwr_ioctl_data));
	if (addr_width > 0) { /* Offset inside device was specified */
		i2c_translate_offset(offset, addr_width, msgbuf0);
		msg_rdwr.msgs[0].addr = addr;
		msg_rdwr.msgs[0].len = addr_width;
		msg_rdwr.msgs[0].flags = 0;
		msg_rdwr.msgs[0].buf = msgbuf0;
		msg_rdwr.msgs[1].addr = addr;
		msg_rdwr.msgs[1].len = datalen;
		msg_rdwr.msgs[1].flags = I2C_M_RD;
		msg_rdwr.msgs[1].buf = msgbuf1;
		msg_rdwr.nmsgs = 2;
	}
	else { /* Offset not specified */
		msg_rdwr.msgs[0].addr = addr;
		msg_rdwr.msgs[0].len = datalen;
		msg_rdwr.msgs[0].flags = I2C_M_RD;
		msg_rdwr.msgs[0].buf = msgbuf1;
		msg_rdwr.nmsgs = 1;
	}
	msg_rdwr.bus = bus;

	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_I2C_READ, &msg_rdwr);
	close(fd);

	DBG("I2C read bus=%d addr=0x%x offset=0x%x addr_width=%zd datalen=%zd rc=%d, data=0x%x\n",
		   bus, addr, offset, addr_width, datalen, rc, msgbuf1[0]);

	memcpy(data, msgbuf1, datalen);

	return rc > 0 ? 0 : rc;
}

int mlnx_i2c_write(int bus, u16 addr, u32 offset, size_t addr_width, char *data, size_t datalen)
{
	unsigned char msgbuf[I2C_BLOCK_MAX];
	struct mlnx_rdwr_i2c_ioctl_params msg_rdwr;
	char path[256];
	int fd;
	int rc;

	if (addr_width + datalen > I2C_BLOCK_MAX)
		return -ENOSPC;
	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);

	memset(&msg_rdwr, 0, sizeof(struct mlnx_rdwr_i2c_ioctl_params));
	memset(msgbuf, 0, sizeof(struct i2c_rdwr_ioctl_data));
	if (addr_width > 0) {
		i2c_translate_offset(offset, addr_width, msgbuf);
		//memcpy(&msgbuf[addr_width], data, datalen);
	}
	memcpy(&msgbuf[addr_width], data, datalen);

	msg_rdwr.msgs[0].addr = addr;
	msg_rdwr.msgs[0].flags = 0;
	msg_rdwr.msgs[0].len = addr_width + datalen;
	msg_rdwr.msgs[0].buf = msgbuf;
	msg_rdwr.nmsgs = 1;
	msg_rdwr.bus = bus;

	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_I2C_WRITE, &msg_rdwr);
	close(fd);

	DBG("I2C write bus=%d addr=0x%x offset=0x%x addr_width=%zd datalen=%zd rc=%d data=0x%lx\n",
		  bus, addr, offset, addr_width, datalen, rc, (unsigned long)data[0]);

	return rc > 0 ? 0 : rc;
}

//#endif

int mlnx_read_vpd(int bus, u16 addr, u32 offset, size_t addr_width, char* data, size_t datalen)
{
	unsigned char msgbuf0[4];
	unsigned char msgbuf1[I2C_CHUNK_MAX]; /* I2C_CHUNK_MAX, since data and control are provided in separate buffers */
	struct mlnx_rdwr_i2c_ioctl_params msg_rdwr;
	char path[256];
	int fd;
	int rc = 0;
	size_t len;


	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);

	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;

	len = datalen;
	while (len > 0) {
		memset(&msg_rdwr, 0, sizeof(struct mlnx_rdwr_i2c_ioctl_params));
		memset(msgbuf1, 0, sizeof(struct i2c_rdwr_ioctl_data));
		if (((offset % I2C_CHUNK_MAX) + len) > I2C_CHUNK_MAX) {
			if (len <= I2C_CHUNK_MAX)
				datalen = I2C_CHUNK_MAX - (offset % I2C_CHUNK_MAX);
			else
				datalen = (I2C_CHUNK_MAX - (offset % I2C_CHUNK_MAX)) % I2C_CHUNK_MAX;
		}
		DBG("I2C read VPD offset=0x%x datalen=%zd len=%zd, data=%p\n",
		       offset, datalen, len, data);
		if (addr_width > 0) { /* Offset inside device was specified */
			i2c_translate_offset(offset, addr_width, msgbuf0);
			msg_rdwr.msgs[0].addr = addr;
			msg_rdwr.msgs[0].len = addr_width;
			msg_rdwr.msgs[0].flags = 0;
			msg_rdwr.msgs[0].buf = msgbuf0;
			msg_rdwr.msgs[1].addr = addr;
			msg_rdwr.msgs[1].len = datalen;
			msg_rdwr.msgs[1].flags = I2C_M_RD;
			msg_rdwr.msgs[1].buf = msgbuf1;
			msg_rdwr.nmsgs = 2;
		}
		else { /* Offset not specified */
			msg_rdwr.msgs[0].addr = addr;
			msg_rdwr.msgs[0].len = datalen;
			msg_rdwr.msgs[0].flags = I2C_M_RD;
			msg_rdwr.msgs[0].buf = msgbuf1;
			msg_rdwr.nmsgs = 1;
		}
		msg_rdwr.bus = bus;
		rc = ioctl(fd, IOCTL_MLNX_I2C_READ, &msg_rdwr);
		if (rc <= 0)
			break;
		memcpy(data, msgbuf1, datalen);
		len -= datalen;
		offset += datalen;
		data += datalen;
		datalen = len;

		DBG("I2C read VPD bus=%d addr=0x%x offset=0x%x addr_width=%zd datalen=%zd rc=%d, data=0x%x\n",
		    bus, addr, offset, addr_width, datalen, rc, msgbuf1[0]);
	}
	close(fd);

	return rc > 0 ? 0 : rc;
}

int mlnx_write_vpd(int bus, u16 addr, u32 offset, size_t addr_width, char *data, size_t datalen)
{
	unsigned char msgbuf[I2C_BLOCK_MAX];/* I2C_BLOCK_MAX, since data and control are provided in same buffer */
	struct mlnx_rdwr_i2c_ioctl_params msg_rdwr;
	char path[256];
	int fd;
	int rc = 0;
	size_t len;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);

	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;

	len = datalen;
	while (len > 0) {
		memset(&msg_rdwr, 0, sizeof(struct mlnx_rdwr_i2c_ioctl_params));
		memset(msgbuf, 0, sizeof(struct i2c_rdwr_ioctl_data));
		if (((offset % I2C_CHUNK_MAX) + len) > I2C_CHUNK_MAX) {
			if (len <= I2C_CHUNK_MAX)
				datalen = I2C_CHUNK_MAX - (offset % I2C_CHUNK_MAX);
			else
				datalen = (I2C_CHUNK_MAX - (offset % I2C_CHUNK_MAX)) % I2C_CHUNK_MAX;
		}
		DBG("I2C write VPD offset=0x%x datalen=%zd len=%zd, data=%p\n",
		     offset, datalen, len, data);
		if (addr_width > 0)
			i2c_translate_offset(offset, addr_width, msgbuf);

		memcpy(&msgbuf[addr_width], data, datalen);

		msg_rdwr.msgs[0].addr = addr;
		msg_rdwr.msgs[0].flags = 0;
		msg_rdwr.msgs[0].len = addr_width + datalen;
		msg_rdwr.msgs[0].buf = msgbuf;
		msg_rdwr.nmsgs = 1;
		msg_rdwr.bus = bus;
		rc = ioctl(fd, IOCTL_MLNX_I2C_WRITE, &msg_rdwr);
		if (rc <= 0)
			break;
		len -= datalen;
		offset += datalen;
		data += datalen;
		datalen = len;
		usleep(VPD_WR_DELAY);

		DBG("I2C write VPD bus=%d addr=0x%x offset=0x%x addr_width=%zd datalen=%zd rc=%d data=0x%lx\n",
		    bus, addr, offset, addr_width, datalen, rc, (unsigned long)data[0]);
	}
	close(fd);

	return rc > 0 ? 0 : rc;
}

int mlnx_read_crspace(int bus, u16 addr, u32 offset, char* data, size_t datalen)
{
	unsigned char msgbuf0[4];
	unsigned char msgbuf1[I2C_CHUNK_MAX]; /* I2C_CHUNK_MAX, since data and control are provided in separate buffers */
	struct mlnx_rdwr_i2c_ioctl_params msg_rdwr;
	char path[256];
	int fd;
	int rc = 0;
	size_t len;
	size_t addr_width = 4;

	if (offset % 4)
		return -1;
	if (offset >= CR_SPACE_SIZE)
		return -1;
	if (datalen > CR_SPACE_CHUNK)
		return -1;
	if (offset + datalen > CR_SPACE_SIZE)
		datalen = CR_SPACE_SIZE - offset;


	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);

	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;

	len = datalen;
	while (len > 0) {
		memset(&msg_rdwr, 0, sizeof(struct mlnx_rdwr_i2c_ioctl_params));
		memset(msgbuf1, 0, sizeof(struct i2c_rdwr_ioctl_data));
		i2c_translate_offset(offset, addr_width, msgbuf0);
		msg_rdwr.msgs[0].addr = addr;
		msg_rdwr.msgs[0].len = addr_width;
		msg_rdwr.msgs[0].flags = 0;
		msg_rdwr.msgs[0].buf = msgbuf0;
		msg_rdwr.msgs[1].addr = addr;
		msg_rdwr.msgs[1].len = datalen;
		msg_rdwr.msgs[1].flags = I2C_M_RD;
		msg_rdwr.msgs[1].buf = msgbuf1;
		msg_rdwr.nmsgs = 2;
		msg_rdwr.bus = bus;
		rc = ioctl(fd, IOCTL_MLNX_I2C_READ, &msg_rdwr);
		if (rc <= 0)
			break;
		memcpy(data, msgbuf1, datalen);
		len -= datalen;
		offset += datalen;
		data += datalen;
		datalen = len;
		DBG("I2C read VPD bus=%d addr=0x%x offset=0x%x addr_width=%zd datalen=%zd rc=%d, data=0x%x\n",
		    bus, addr, offset, addr_width, datalen, rc, msgbuf1[0]);
	}
	close(fd);

	return rc > 0 ? 0 : rc;
}

int mlnx_i2c_read_attr(const char *attr, int bus, int addr, int offset, int length, char* data)
{
	char path[256];
	FILE *fp;
	int rc;

	snprintf(path, sizeof(path), SYSFS_PATH_ATTR, bus, addr, attr);
	fp = fopen(path, "r+");
	if (NULL == fp) {
		fp = fopen(path, "r");
		if (NULL == fp)
			return -EFAULT;
	}
	rc = fseek(fp, offset, SEEK_SET);
	if (rc < 0)  {
		fclose(fp);
		return -EFAULT;
	}
	rc = fread (data, 1, length, fp);
	fclose(fp);

	return rc;
}

int mlnx_i2c_read_attr_cached(const char *attr, int bus, int addr, int offset, int length, char* data)
{
	char path[256];
	char path_cache[256];
	FILE *fp;
	FILE *fp_cache;
	int rc;

	snprintf(path, sizeof(path), SYSFS_PATH_ATTR, bus, addr, attr);
	snprintf(path_cache, sizeof(path), SYSFS_PATH_ATTR, bus, addr, "cache");
	fp_cache = fopen(path_cache, "w");
	if (NULL == fp_cache)
		return -EFAULT;
	rc = fwrite (data, 1, length, fp_cache);
	fclose(fp_cache);
	fp = fopen(path, "r+");
	if (NULL == fp)
		return -EFAULT;
	if (rc < 0) {
		fclose(fp);
		return -EFAULT;
	}
	rc = fseek(fp, offset, SEEK_SET);
	if (rc < 0) {
		fclose(fp);
		return -EFAULT;
	}
	rc = fread (data, 1, length, fp);
	fclose(fp);

	return rc;
}

int mlnx_i2c_write_attr(const char *attr, int bus, int addr, int offset, int length, char* data)
{
	char path[256];
	FILE *fp;
	int rc;

	snprintf(path, sizeof(path), SYSFS_PATH_ATTR, bus, addr, attr);
	fp = fopen(path, "r+");
	if (NULL == fp)
		return -EFAULT;

	rc = fseek(fp, offset, SEEK_SET);
	if (rc < 0)  {
		fclose(fp);
		return -EFAULT;
	}
	rc = fwrite (data, 1, length, fp);
	fclose(fp);

	return rc;
}

int mlnx_add_i2c_virtual_buses(struct mlnx_i2c_virt_bus_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	DBG("add to bus %d %d elements addr=%x\n",
		  params->parent_bus_num, params->element_num, params->sw[0].i2c_dev_addr);

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_ADD_I2C_VIRTUAL_BUSES, params);
	DBG("rc=%d \n", rc);
	close(fd);

	return rc;
}

int mlnx_delete_i2c_virtual_buses(struct mlnx_i2c_switch_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_DELETE_I2C_VIRTUAL_BUSES, params);
	close(fd);

	return rc;
}

int mlnx_block_i2c_virtual_buses(struct mlnx_i2c_switch_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_BLOCK_I2C_VIRTUAL_BUSES, params);
	close(fd);

	return rc;
}

int mlnx_unblock_i2c_virtual_buses(struct mlnx_i2c_switch_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_UNBLOCK_I2C_VIRTUAL_BUSES, params);
	close(fd);

	return rc;
}

int mlnx_reset_i2c_virtual_buses_and_devices(struct mlnx_i2c_switch_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_RESET_I2C_VIRTUAL_BUSES_AND_DEVICES, params);
	close(fd);

	return rc;
}

int mlnx_add_devices_to_vbus(struct mlnx_i2c_devices_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_ADD_I2C_DEVICES_TO_VBUS, params);
	close(fd);

	return rc;
}

int mlnx_delete_devices_from_vbus(struct mlnx_i2c_switch_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_DELETE_I2C_DEVICES_FROM_VBUS, params);
	close(fd);

	return rc;
}

int mlnx_test_vbus_exist(struct mlnx_i2c_virt_bus_scan_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_TEST_I2C_VBUS, params);
	close(fd);

	return rc;
}

int mlnx_test_device_exist(struct mlnx_i2c_one_device_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_TEST_I2C_DEV, params);
	close(fd);

	return rc;
}

int mlnx_scan_switch_devices(struct mlnx_i2c_virt_bus_scan_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_SCAN_I2C_SWITCH_DEVICES, params);
	close(fd);

	return rc;
}

int mlnx_add_gpio_i2c_buses(struct mlnx_gpio_i2c_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	DBG("add to bus %d gpio muxes group\n", params->element_num);

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_ADD_I2C_GPIO_VIRTUAL_BUSES, params);
	close(fd);

	return rc;
}

int mlnx_del_gpio_i2c_buses(struct mlnx_gpio_i2c_del_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_DELETE_I2C_GPIO_VIRTUAL_BUSES, params);
	close(fd);

	return rc;
}

int mlnx_gpio_read(struct mlnx_gpio_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_GPIO_READ, params);
	close(fd);

	return rc;
}

int mlnx_gpio_write(struct mlnx_gpio_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_GPIO_WRITE, params);
	close(fd);

	return rc;
}

int mlnx_gpio_set_dir(struct mlnx_gpio_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_GPIO_SET_DIR, params);
	close(fd);

	return rc;
}

int mlnx_add_one_device_to_vbus(struct mlnx_i2c_one_device_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_ADD_I2C_ONE_DEVICE_TO_VBUS, params);
	close(fd);

	return rc;
}

int mlnx_delete_one_device_from_vbus(struct mlnx_i2c_one_device_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_DELETE_I2C_ONE_DEVICE_FROM_VBUS, params);
	close(fd);

	return rc;
}






int mlnx_cpld_irq_subscribe(struct mlnx_cpld_subscribe_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_CPLD_IRQ_SUBSCRIBE, params);
	close(fd);

	return rc;
}

int mlnx_cpld_irq_unsubscribe(struct mlnx_cpld_subscribe_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_CPLD_IRQ_UNSUBSCRIBE, params);
	close(fd);

	return rc;
}

int mlnx_cpld_irq_set(struct mlnx_cpld_set_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_CPLD_IRQ_SET, params);
	close(fd);

	return rc;
}

int mlnx_cpld_irq_get_data(struct mlnx_cpld_get_data_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_CPLD_IRQ_GET_DATA, params);
	close(fd);

	return rc;
}

int mlnx_watchdog_config(struct mlnx_watchdog_config_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_WATCHDOG_CONFIG, params);
	close(fd);

	return rc;
}

int mlnx_watchdog_enable_disable(struct mlnx_watchdog_enable_disable_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_WATCHDOG_ENA_DIS, params);
	close(fd);

	return rc;
}

int mlnx_cpld_leds_config(struct mlnx_cpld_leds_config_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME_UNBLOCKED);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_CPLD_CONFIG_LED, params);
	close(fd);

	return rc;
}

int mlnx_asic_fan_config(struct mlnx_asic_fan_config_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME_UNBLOCKED);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_ASIC_CONFIG_FAN, params);
	close(fd);

	return rc;
}

int mlnx_asic_temp_config(struct mlnx_asic_temp_sensor_config_ioctl_params *params)
{
	char path[256];
	int fd;
	int rc = 0;

	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME_UNBLOCKED);
	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_ASIC_CONFIG_TEMP, params);
	close(fd);

	return rc;
}

int mlnx_i2c_read_smbus_legacy(int bus, u16 addr, u32 offset, size_t addr_width, char *data, size_t datalen)
{
	struct i2c_smbus_ioctl_data msg_rdwr_smbus;
	union i2c_smbus_data __user rwdata;
	char path[256];
	int fd;
	int rc;

	if (datalen > I2C_SMBUS_BLOCK_MAX)
		return -ENOSPC;

	sprintf(path, "%s%d/%d-%04x/eeprom", "/sys/bus/i2c/devices/i2c-", bus, bus, addr);
	memset(&msg_rdwr_smbus, 0, sizeof(struct i2c_smbus_ioctl_data));
	memset(&rwdata, 0, sizeof(union i2c_smbus_data));
	msg_rdwr_smbus.data = &rwdata;
	if (addr_width > 0) { /* Offset inside device was specified */
		if (datalen > 2) {
			msg_rdwr_smbus.data->block[0] = datalen; /* block[0] is used for length */
			msg_rdwr_smbus.size = I2C_SMBUS_I2C_BLOCK_DATA;//I2C_SMBUS_BLOCK_DATA;
		} else if (datalen == 2) {
			msg_rdwr_smbus.size = I2C_SMBUS_WORD_DATA;
		} else if (datalen == 1) {
			msg_rdwr_smbus.size = I2C_SMBUS_BYTE_DATA;
		}
	}
	else { /* Offset not specified */
		msg_rdwr_smbus.size = I2C_SMBUS_BYTE;
	}
	msg_rdwr_smbus.read_write = I2C_SMBUS_READ;
	msg_rdwr_smbus.command = offset;

	DBG("SMBUS read (%d) path=%s bus=%d addr=0x%x offset=0x%x addr_width=%zd datalen=%zd command=%x size=%x byte 0x%x word 0x%x block 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x\n",
		   __LINE__, path, bus, addr, offset, addr_width, datalen, msg_rdwr_smbus.command,
		   msg_rdwr_smbus.size, msg_rdwr_smbus.data->byte,
		   msg_rdwr_smbus.data->word, msg_rdwr_smbus.data->block[0],
		   msg_rdwr_smbus.data->block[1], msg_rdwr_smbus.data->block[2], msg_rdwr_smbus.data->block[3],
		   msg_rdwr_smbus.data->block[4], msg_rdwr_smbus.data->block[5]);

	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, I2C_SMBUS, &msg_rdwr_smbus);
	close(fd);

	DBG("SMBUS read (%d) path=%s bus=%d addr=0x%x offset=0x%x addr_width=%zd datalen=%zd rc=%d command=%x byte 0x%x word 0x%x block 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x\n",
		   __LINE__, path, bus, addr, offset, addr_width, datalen, rc, msg_rdwr_smbus.command,
		   msg_rdwr_smbus.data->byte, msg_rdwr_smbus.data->word, msg_rdwr_smbus.data->block[0],
		   msg_rdwr_smbus.data->block[1], msg_rdwr_smbus.data->block[2], msg_rdwr_smbus.data->block[3],
		   msg_rdwr_smbus.data->block[4], msg_rdwr_smbus.data->block[5]);

	if (datalen > 2) {
		memcpy(data, &msg_rdwr_smbus.data->block[1], msg_rdwr_smbus.data->block[0]);
	} else if (datalen == 2) {
		memcpy(data, &msg_rdwr_smbus.data->word, 2);
	} else if (datalen == 1) {
		memcpy(data, &msg_rdwr_smbus.data->byte, 1);
	}
	return rc > 0 ? 0 : rc;
}

int mlnx_i2c_write_smbus_legacy(int bus, u16 addr, u32 offset, size_t addr_width, char *data, size_t datalen)
{
	struct i2c_smbus_ioctl_data msg_rdwr_smbus;
	union i2c_smbus_data __user rwdata;
	char path[256];
	int fd;
	int rc;

	if (datalen > I2C_SMBUS_BLOCK_MAX)
		return -ENOSPC;

	sprintf(path, "%s%d", "/dev/i2c-", bus);
	memset(&msg_rdwr_smbus, 0, sizeof(struct i2c_smbus_ioctl_data));
	memset(&rwdata, 0, sizeof(union i2c_smbus_data));
	msg_rdwr_smbus.data = &rwdata;
	if (addr_width > 0) { /* Offset inside device was specified */
		if (datalen > 2) {
			msg_rdwr_smbus.data->block[0] = datalen; /* block[0] is used for length */
			msg_rdwr_smbus.size = I2C_SMBUS_I2C_BLOCK_DATA;//I2C_SMBUS_BLOCK_DATA
			memcpy(&msg_rdwr_smbus.data->block[1], data, datalen);
		} else if (datalen == 2) {
			msg_rdwr_smbus.size = I2C_SMBUS_WORD_DATA;
			memcpy(&msg_rdwr_smbus.data->word, data, datalen);
		} else if (datalen == 1) {
			msg_rdwr_smbus.size = I2C_SMBUS_BYTE_DATA;
			memcpy(&msg_rdwr_smbus.data->byte, data, datalen);
		}
	}
	else { /* Offset not specified */
		msg_rdwr_smbus.size = I2C_SMBUS_BYTE;
	}
	msg_rdwr_smbus.read_write = I2C_SMBUS_WRITE;
	msg_rdwr_smbus.command = offset;

	DBG("SMBUS write path=%s bus=%d addr=0x%x offset=0x%x addr_width=%zd datalen=%zd command=%x size=%x byte 0x%x word 0x%x block 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x\n",
		   path, bus, addr, offset, addr_width, datalen, msg_rdwr_smbus.command, msg_rdwr_smbus.size,
		   msg_rdwr_smbus.data->byte, msg_rdwr_smbus.data->word, msg_rdwr_smbus.data->block[0],
		   msg_rdwr_smbus.data->block[1], msg_rdwr_smbus.data->block[2], msg_rdwr_smbus.data->block[3],
		   msg_rdwr_smbus.data->block[4], msg_rdwr_smbus.data->block[5]);

	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, I2C_SMBUS, &msg_rdwr_smbus);
	close(fd);

	DBG("SMBUS write path=%s bus=%d addr=0x%x offset=0x%x addr_width=%zd datalen=%zd rc=%d command=%x byte 0x%x word 0x%x block 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x\n",
		   path, bus, addr, offset, addr_width, datalen, rc, msg_rdwr_smbus.command,
		   msg_rdwr_smbus.data->byte, msg_rdwr_smbus.data->word, msg_rdwr_smbus.data->block[0],
		   msg_rdwr_smbus.data->block[1], msg_rdwr_smbus.data->block[2], msg_rdwr_smbus.data->block[3],
 		   msg_rdwr_smbus.data->block[4], msg_rdwr_smbus.data->block[5]);

	return rc > 0 ? 0 : rc;
}

//////////////////////////////////////////////////////////////////////////////
// Function name : smbus_eeprom_read
// Description   : Read buflen bytes from the EEPROM beginning at start_offset
// Return type   : 0 for success, -1 for failure
// Argument      : int fd : File descriptor returned by open()
// Argument      : unsigned char start_offset : Read bytes starting at this
//                     offset in the EEPROM.  The sum of buflen and
//                     start_offset must not exceed the maximum size in bytes
//                     of the EEPROM
// Argument      : unsigned char *buffer : Where to store the bytes read
//                     from the EEPROM.  The buffer must be large enough
//                     to store buflen bytes read from the EEPROM.
// Argument      : unsigned short buflen : The size in bytes of buffer, or
//                     how many bytes to read from the EEPROM.  The sum of
//                     buflen and start_offset must not exceed the maximum
//                     size in bytes of the EEPROM.
// Argument      : eeprom_size
static int smbus_eeprom_read(int fd, unsigned char start_offset, unsigned char *buffer,
				unsigned short buflen, int eeprom_size)
{
	struct i2c_smbus_ioctl_data msg_rdwr_smbus;
	union i2c_smbus_data __user rwdata;
        int offset, ind, rc;
	memset(&msg_rdwr_smbus, 0, sizeof(struct i2c_smbus_ioctl_data));
	memset(&rwdata, 0, sizeof(union i2c_smbus_data));
	msg_rdwr_smbus.data = &rwdata;
	msg_rdwr_smbus.size = I2C_SMBUS_BYTE_DATA;
	msg_rdwr_smbus.read_write = I2C_SMBUS_READ;
        for (ind = 0, offset = start_offset; ind < buflen &&
             offset < eeprom_size; ind++, offset++) {
		msg_rdwr_smbus.command = offset;
                rc = ioctl(fd, I2C_SMBUS, &msg_rdwr_smbus);
                if (rc < 0) {
                        printf("ERROR: i2c_smbus_read_byte_data(fd, 0x%02X) failed\n", offset);
                        printf("errno = %d, %s\n", errno, strerror(errno));
                        return -1;
                }
                buffer[ind] = (unsigned char)(msg_rdwr_smbus.data->byte);
        }
	DBG("SMBUS read (%d) fd=%d offset=0x%x len=%d buf 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x\n",
	     __LINE__, fd, start_offset, buflen, buffer[0], buffer[1],
	     buffer[2], buffer[3], buffer[4], buffer[5], buffer[6], buffer[7], buffer[8]);
        return 0;
}

//////////////////////////////////////////////////////////////////////////////
// Function name : smbus_eeprom_write
// Description   : Write buflen bytes to the EEPROM beginning at start_offset
// Return type   : 0 for success, -1 for failure
// Argument      : int fd : File descriptor returned by open()
// Argument      : unsigned char start_offset : Write bytes starting at this
//                     offset in the EEPROM.  The sum of buflen and
//                     start_offset must not exceed the maximum size in bytes
//                     of the EEPROM
// Argument      : unsigned char *buffer : Where to get the bytes to write
//                     to the EEPROM.
// Argument      : unsigned short buflen : The size in bytes of buffer.
//                     The sum of buflen and start_offset must not exceed the
//                     maximum size in bytes of the EEPROM.
// Argument      : eeprom_size
static int smbus_eeprom_write(int fd, unsigned char start_offset, unsigned char *buffer,
				unsigned short buflen, int eeprom_size)
{
	struct i2c_smbus_ioctl_data msg_rdwr_smbus;
	union i2c_smbus_data __user rwdata;
	int offset, ind, rc;
	memset(&msg_rdwr_smbus, 0, sizeof(struct i2c_smbus_ioctl_data));
	memset(&rwdata, 0, sizeof(union i2c_smbus_data));
	DBG("SMBUS write (%d) fd=%d offset=0x%x len=%d buf 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x\n",
		   __LINE__, fd, start_offset, buflen, buffer[0], buffer[1],
		   buffer[2], buffer[3], buffer[4], buffer[5], buffer[6], buffer[7], buffer[8]);
	msg_rdwr_smbus.data = &rwdata;
	msg_rdwr_smbus.size = I2C_SMBUS_BYTE_DATA;
	msg_rdwr_smbus.read_write = I2C_SMBUS_WRITE;
	for (ind = 0, offset = start_offset; ind < buflen &&
	     offset < eeprom_size; ind++, offset++) {
		msg_rdwr_smbus.command = offset;
		msg_rdwr_smbus.data->byte = buffer[ind];
        	rc = ioctl(fd, I2C_SMBUS, &msg_rdwr_smbus);
        	if (rc < 0) {
			printf("ERROR: i2c_smbus_write_byte_data(fd, 0x%02X, 0x%02X) failed\n",
				offset, buffer[ind]);
			printf("errno = %d, %s\n", errno, strerror(errno));
			return -1;
		}
		/* Delay while the byte write completes */
		usleep(100);
	}
	return 0;
}

/*
    adapter_num  To match with /dev/i2c-0
    i2c_addr     SMBus address of EEPROM
    offset       Which byte to access in the EEPROM
    data_size    No more than EEPROM_SIZE
*/
int eeprom_rw(int adapter_num, int i2c_addr, unsigned short offset, unsigned short data_size, int eeprom_size)
{
    char path[256];
    int fd;
    int rc;
    unsigned char rbuffer[16]; /* Data read from EEPROM */
    unsigned char wbuffer[16] = {
        /* Data written to EEPROM */
        0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17,
        0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F
    };
    /* Open the special device file for the SMBus */
    sprintf(path, "/dev/i2c-%d", adapter_num);
    fd = open(path, O_RDWR);
    if (fd < 0) {
        printf("ERROR: open(%s) failed\n", path);
        printf("errno = %d, %s\n", errno, strerror(errno));
        return -1;
    }
    printf("SUCCESS: open(%s) passed\n", path);
    memset(rbuffer, 0, 16);
    /* Specify the EEPROM as the device we want to access.
       *** IMPORTANT ***
       The address is actually in the 7 LSBs, so shift
       i2c_addr one bit to the right.*/
    rc = ioctl(fd, I2C_SLAVE, i2c_addr>>1);
    if (rc < 0) {
        printf("ERROR: ioctl(fd, I2C_SLAVE, 0x%02X) failed\n", i2c_addr);
        printf("errno = %d, %s\n", errno, strerror(errno));
        close(fd);
        return -1;
    }
    printf("SUCCESS: ioctl(fd, I2C_SLAVE, 0x%02X>>1) passed\n", i2c_addr);
    /* Read data from the EEPROM */
    offset = 0;
    if (smbus_eeprom_read(fd, offset, rbuffer, data_size, eeprom_size) < 0) {
        printf("ERROR: smbus_eeprom_read() failed\n");
        close(fd);
        return -1;
    }
    /* Dump the data read from the EEPROM */
    for (offset=0; offset<data_size; offset++)
        printf("Offset: %02d   Data: 0x%02X\n", offset, rbuffer[offset]);
    /* Write data to the EEPROM */
    printf("\nPress ENTER to write to the EEPROM");
    getchar();
    offset = 0;
    if (smbus_eeprom_write(fd, offset, wbuffer, data_size, eeprom_size) < 0) {
        printf("ERROR: smbus_eeprom_write() failed\n");
        close(fd);
        return -1;
    }
    printf("SUCCESS: Data written to the EEPROM\n");
    /* Read data again from the EEPROM */
    printf("\nPress ENTER to read again from the EEPROM");
    getchar();
    offset = 0;
    if (smbus_eeprom_read(fd, offset, rbuffer, data_size, eeprom_size) < 0) {
        printf("ERROR: smbus_eeprom_read() failed\n");
        close(fd);
        return -1;
    }
    /* Dump the data read from the EEPROM */
    for (offset=0; offset < data_size; offset++)
        printf("Offset: %02d   Data: 0x%02X\n", offset, rbuffer[offset]);
    printf("\n");
    /* Close the special device file */
    close(fd);
    return 0;
}

int mlnx_vpd_rw_limited(struct mlnx_rdwr_vpd_i2c_ioctl_params *params)
{
	struct mlnx_rdwr_vpd_i2c_ioctl_params msg_rdwr;
	char path[256];
	int fd;
	int cmd;
	int rc;

	if (params->datalen > I2C_BLOCK_MAX)
		return -ENOSPC;
	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);

	if (params->flags == 0) {
		memcpy(msg_rdwr.databuf, params->databuf, params->datalen);
		cmd = IOCTL_MLNX_VPD_WRITE;
	}
	else {
		cmd = IOCTL_MLNX_VPD_READ;
	}
	msg_rdwr.bus = params->bus;
	msg_rdwr.addr = params->addr;
	msg_rdwr.addr_width = params->addr_width;
	msg_rdwr.flags = params->flags;
	msg_rdwr.offset = params->offset;
	msg_rdwr.datalen = params->datalen;
	msg_rdwr.bus_prot = params->bus_prot;
	msg_rdwr.addr_prot = params->addr_prot;
	msg_rdwr.reg_prot = params->reg_prot;
	msg_rdwr.get_prot = params->get_prot;
	msg_rdwr.put_prot = params->put_prot;
	msg_rdwr.limit = params->limit;
	msg_rdwr.border = params->border;
	msg_rdwr.read_chunk_delay = params->read_chunk_delay;
	msg_rdwr.write_chunk_delay = params->write_chunk_delay;
	msg_rdwr.post_delay = params->post_delay;

	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, cmd, &msg_rdwr);
	close(fd);

	DBG("I2C read/write bus=%d addr=%hx offset=%x addr_width=%d bus_prot=%d addr_prot=%x reg_prot=%x get_prot=%x put_prot=%x limit=%d border=%d rchunk_del=%d wchunk_del=%d post_del=%d flags=%x datalen=%d rc=%d\n",
		   params->bus, params->addr, params->offset, params->addr_width,
		   params->bus_prot, params->addr_prot, params->reg_prot, params->get_prot,
		   params->put_prot, params->limit, params->border, params->read_chunk_delay,
		   params->write_chunk_delay, msg_rdwr.post_delay, params->flags, params->datalen, rc);
	if (params->flags == 1)
		memcpy(params->databuf, msg_rdwr.databuf, params->datalen);

	return rc > 0 ? 0 : rc;
}

int mlnx_lpc_rw(u16 base, u16 offset, u8 read_write, char *data, size_t datalen)
{
	struct mlnx_lpc_ioctl_params msg_rdwr;
	char path[256];
	int fd;
	int rc;

	if (datalen > LPCI2C_DATA_REG_SZ)
		return -ENOSPC;
	if (!datalen)
		return -EINVAL;
	snprintf(path, sizeof(path), MLNX_IF_DEVICE_NAME);

	msg_rdwr.base = base;
	msg_rdwr.offset = offset;
	msg_rdwr.read_write = read_write;
	msg_rdwr.datalen = datalen;
	if (read_write == 0)
		memcpy(msg_rdwr.data, data, datalen);

	fd = open(path, O_RDWR|O_SYNC);
	if (fd < 0)
		return -EFAULT;
	rc = ioctl(fd, IOCTL_MLNX_LPC_RW, &msg_rdwr);
	close(fd);
	if (read_write == 1)
        memcpy(data, msg_rdwr.data, datalen);

	return rc > 0 ? 0 : rc;
}
